/* This file is auto generated, version 1 */
/* SMP */
#define UTS_MACHINE "i386"
#define UTS_VERSION "#1 SMP Sat, 01 Aug 2026 15:50:20 +0000"
#define LINUX_COMPILE_BY "linux-lts515"
#define LINUX_COMPILE_HOST "archlinux"
#define LINUX_COMPILER "gcc (GCC) 15.2.1 20260209, GNU ld (GNU Binutils) 2.46"
