/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_X86_IA32_UNISTD_H
#define _ASM_X86_IA32_UNISTD_H

/*
 * This file contains the system call numbers of the ia32 compat ABI,
 * this is for the kernel only.
 */
#define __SYSCALL_ia32_NR(x) (x)
#include <asm/unistd_32_ia32.h>

#endif /* _ASM_X86_IA32_UNISTD_H */
