/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef __ASM_GENERIC_MODULE_LDS_H
#define __ASM_GENERIC_MODULE_LDS_H

/*
 * <asm/module.lds.h> can specify arch-specific sections for linking modules.
 * Empty for the asm-generic header.
 */

#endif /* __ASM_GENERIC_MODULE_LDS_H */
